#include <LiquidCrystal.h>
#include <Wire.h>
#include <Ultrasonic.h>

Ultrasonic sensor_B(3, 2);
Ultrasonic sensor_F(4, 5);
LiquidCrystal lcd(13, 12, 11, 10, 9, 8);

#define IMPACT_DIST 4
#define BEGIN_DIST 34

#define BUZZER 6

bool hit = 0, safe_range = 1;
int pause_time, break_millis;

void setup() {
  Serial.begin(9600);
  lcd.begin(16, 2);
  pinMode(6, OUTPUT);
  DDRD |= (1 << PD7); // led
}
  
void print_dist_serial(float back, float front) {
  Serial.print("Back: ");
  Serial.println(back);

  Serial.print("Front: ");
  Serial.println(front);
}

void print_dist_lcd(float back, float front) {
  if (safe_range == 0 && millis() - break_millis < 300) {
    return;
  }
  
  lcd.clear();
  if (back > BEGIN_DIST)
    lcd.print("N/A");
  else if (back < IMPACT_DIST)
    lcd.print("HIT");
  else
    lcd.print(back);
  
  lcd.setCursor(0, 1); // set the cursor to column 0, line 2
  
  if (front > BEGIN_DIST)
    lcd.print("N/A");
  else if (front < IMPACT_DIST)
    lcd.print("HIT");
  else
    lcd.print(front);

  lcd.setCursor(0, 0);// set the cursor to column 0, line1
}

bool buzz(float left, float right) {
  float min_val = min(left, right);
  if (min_val > BEGIN_DIST) {
    safe_range = 1;
    return false;
  }
  int pause_time = 10.5 * pow(2, 1.5 * min_val / 10) - 10.5;
  
  analogWrite(BUZZER, 127);
  if (min_val > 7) {
    safe_range = 1;
    delay(100);
    analogWrite(BUZZER, 0);
    delay(pause_time);
  } else {
    safe_range = 0;
    break_millis = millis();
  }
  return true;
}

void loop() {
  long microsec_B = sensor_B.timing(), microsec_F = sensor_F.timing();
  float left_dist = microsec_B * 0.034 / 2;
  float right_dist = microsec_F * 0.034 / 2;
  
  print_dist_serial(left_dist, right_dist);
  
  if ((left_dist < IMPACT_DIST || right_dist < IMPACT_DIST) && hit == 0) {
    PORTD |= (1 << PD7);
    hit = 1;
  }

  if (buzz(left_dist, right_dist) == false)
    delay(300);

  print_dist_lcd(left_dist, right_dist);
}
